import { defaultConfig as defaultConfig$1 } from '@cleavelandprice/ngx-lib';
import * as i0 from '@angular/core';
import { Injectable, Optional, Pipe, Component, Input, NgModule, SkipSelf } from '@angular/core';
import { of, map } from 'rxjs';
import moment from 'moment';
import * as i2 from '@angular/common/http';
import * as i1 from '@cleavelandprice/ngx-lib/active-directory';
import * as i2$1 from '@angular/common';
import { CommonModule } from '@angular/common';

const defaultConfig = {
    apiUrl: defaultConfig$1.apiUrl,
    usersEndpoint: '/sharepoint/users',
    listItemsEndpoint: '/sharepoint/listitems',
    username: 'viewer',
    password: 'viewer',
    photos: {
        site: 'https://sharepoint.cleavelandprice.com/digitaldisplay',
        list: 'Employee Thumbnail Photos',
        additionalPropertiesToRetrieve: [
            'EncodedAbsUrl'
        ]
    },
    caching: {
        enabled: true,
        lifeMinutes: 720
    },
    logging: {
        dataCached: true,
        cachedDataReturned: true,
        freshDataReturned: true,
        cacheCleared: true
    }
};

class ServiceConfig {
}

class SharePointService {
    //#endregion
    //#region Initialization
    // If a ServiceConfig object is not provided when the module is imported, then default values are pulled in from default-config.ts.
    constructor(config, http) {
        this.config = config;
        this.http = http;
        this.cache = null;
        this.defaultCacheLifeMinutes = 720;
        this.config = {
            ...defaultConfig,
            ...this.config
        };
        this.configureCaching();
        this.configureLogging();
    }
    //#region Utilities
    cacheExpired(cache) {
        if (!cache) {
            return true;
        }
        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
        const expiration = moment(cache.lastUpdate)
            .add(lifeMinutes, 'minutes');
        return moment().isAfter(expiration);
    }
    configureCaching() {
        // If caching config is missing entirely, give it a default
        this.config.caching = this.config.caching ?? {
            enabled: false,
            lifeMinutes: this.defaultCacheLifeMinutes
        };
        this.cache = {
            photos: null,
            users: null
        };
    }
    configureLogging() {
        // If logging config is missing entirely, give it a default
        this.config.logging = this.config.logging ?? {
            dataCached: false,
            cachedDataReturned: false,
            freshDataReturned: false,
            cacheCleared: false
        };
    }
    //#endregion
    getListItems(request) {
        const apiUrl = this.config?.apiUrl ?? defaultConfig.apiUrl;
        const endpoint = this.config?.listItemsEndpoint ?? defaultConfig.listItemsEndpoint;
        return this.http.post(`${apiUrl}${endpoint}`, request);
    }
    getPhotos() {
        const request = {
            username: this.config?.username ?? defaultConfig.username,
            password: this.config?.password ?? defaultConfig.password,
            site: this.config?.photos.site ?? defaultConfig.photos.site,
            list: this.config?.photos.list ?? defaultConfig.photos.list,
            additionalPropertiesToRetrieve: defaultConfig.photos.additionalPropertiesToRetrieve
        };
        const observable$ = this.getListItems(request);
        if (this.config.caching?.enabled) {
            if (this.cacheExpired(this.cache?.photos)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.photos = {
                        data,
                        lastUpdate: moment()
                    };
                    if (this.config.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Photos cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached photos');
                }
                return of(this.cache.photos.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config.logging?.freshDataReturned) {
            console.log('Returning fresh photos');
        }
        return observable$;
    }
    getUsers(request, id) {
        const apiUrl = this.config?.apiUrl ?? defaultConfig.apiUrl;
        const endpoint = this.config?.usersEndpoint ?? defaultConfig.usersEndpoint;
        const observable$ = this.http.post(`${apiUrl}${endpoint}/${id ?? ''}`, request);
        //#region Single user
        if (id) {
            return observable$;
        }
        //#endregion
        //#region Multiple users
        if (this.config.caching?.enabled) {
            if (this.cacheExpired(this.cache.users)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.users = {
                        data: data,
                        lastUpdate: moment()
                    };
                    if (this.config.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Users cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.cache.users.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config.logging?.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
        //#endregion
    }
    clearCache() {
        this.cache = null;
        if (this.config.logging?.cacheCleared) {
            console.log('Cache cleared');
        }
    }
}
SharePointService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointService, deps: [{ token: ServiceConfig, optional: true }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
SharePointService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiceConfig, decorators: [{
                    type: Optional
                }] }, { type: i2.HttpClient }]; } });

class EmployeePhotoUrlPipe {
    constructor(sharePointService) {
        this.sharePointService = sharePointService;
    }
    transform(user, userPhotos) {
        const observable$ = userPhotos ? of(userPhotos) : this.sharePointService.getPhotos();
        return observable$
            .pipe(map(photos => photos.find(pic => 
        // Try to match by employeeNumber, accountName, or title
        (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()))?.encodedAbsUrl));
        /*const photo = photos
          .find(pic =>
            (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()));
    
        if (!photo) {
          console.log(`No photo match for ${user.displayName}`);
        }
    
        return photo ? photo.encodedAbsUrl : null;*/
    }
}
EmployeePhotoUrlPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: EmployeePhotoUrlPipe, deps: [{ token: SharePointService }], target: i0.ɵɵFactoryTarget.Pipe });
EmployeePhotoUrlPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "14.2.1", ngImport: i0, type: EmployeePhotoUrlPipe, name: "employeePhotoUrl" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: EmployeePhotoUrlPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'employeePhotoUrl'
                }]
        }], ctorParameters: function () { return [{ type: SharePointService }]; } });

class EmployeePhotoComponent {
    constructor(activeDirectoryService) {
        this.activeDirectoryService = activeDirectoryService;
        this.employee = null;
        // Or...
        this.accountName = null;
        this.size = "2em";
        this.notRound = false;
    }
    ngOnInit() {
        if (this.accountName && !this.employee) {
            // Developer passed in an account name instead of a user object
            // We need to do the work of getting the user object ourselves
            this.activeDirectoryService.getUser(this.accountName)
                .subscribe(user => this.employee = user);
        }
    }
}
EmployeePhotoComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: EmployeePhotoComponent, deps: [{ token: i1.ActiveDirectoryService }], target: i0.ɵɵFactoryTarget.Component });
EmployeePhotoComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.1", type: EmployeePhotoComponent, selector: "employee-photo", inputs: { employee: "employee", accountName: "accountName", employeePhotos: "employeePhotos", size: "size", notRound: "notRound" }, ngImport: i0, template: "<ng-container *ngIf=\"employee\">\n    <img *ngIf=\"(employee | employeePhotoUrl:employeePhotos | async)\"\n        [src]=\"employee | employeePhotoUrl:employeePhotos | async\"\n        [style.width]=\"size\"\n        [style.height]=\"size\"\n        [style.border-radius]=\"notRound ? 'default' : '100%'\">\n</ng-container>", styles: [""], dependencies: [{ kind: "directive", type: i2$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i2$1.AsyncPipe, name: "async" }, { kind: "pipe", type: EmployeePhotoUrlPipe, name: "employeePhotoUrl" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: EmployeePhotoComponent, decorators: [{
            type: Component,
            args: [{ selector: 'employee-photo', template: "<ng-container *ngIf=\"employee\">\n    <img *ngIf=\"(employee | employeePhotoUrl:employeePhotos | async)\"\n        [src]=\"employee | employeePhotoUrl:employeePhotos | async\"\n        [style.width]=\"size\"\n        [style.height]=\"size\"\n        [style.border-radius]=\"notRound ? 'default' : '100%'\">\n</ng-container>" }]
        }], ctorParameters: function () { return [{ type: i1.ActiveDirectoryService }]; }, propDecorators: { employee: [{
                type: Input
            }], accountName: [{
                type: Input
            }], employeePhotos: [{
                type: Input
            }], size: [{
                type: Input
            }], notRound: [{
                type: Input
            }] } });

class SharePointModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('SharePointModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(config) {
        return {
            ngModule: SharePointModule,
            providers: [
                SharePointService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
}
SharePointModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointModule, deps: [{ token: SharePointModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
SharePointModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.1", ngImport: i0, type: SharePointModule, declarations: [EmployeePhotoUrlPipe,
        EmployeePhotoComponent], imports: [CommonModule], exports: [EmployeePhotoUrlPipe,
        EmployeePhotoComponent] });
SharePointModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.1", ngImport: i0, type: SharePointModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        EmployeePhotoUrlPipe,
                        EmployeePhotoComponent
                    ],
                    imports: [
                        CommonModule
                    ],
                    exports: [
                        EmployeePhotoUrlPipe,
                        EmployeePhotoComponent
                    ]
                }]
        }], ctorParameters: function () { return [{ type: SharePointModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { EmployeePhotoComponent, EmployeePhotoUrlPipe, ServiceConfig, SharePointModule, SharePointService, defaultConfig };
//# sourceMappingURL=cleavelandprice-ngx-lib-sharepoint.mjs.map
